@@ -4,6 +4,7 @@ import { MentionContext } from './ChatContextMenu/ChatContextMenu';
 import { CachingService, SETTING_KEYS } from '../utils/caching';
 import { STATE_DB_KEYS, StateDBCachingService } from '../utils/stateDBCaching';
 import { AppStateService } from '../AppState';
+import { v4 as uuidv4 } from 'uuid';
 
 export interface IChatThread {
   id: string;
@@ -24,25 +25,28 @@ export interface NotebookChatState {
  * Manager for persisting chat histories across notebook sessions
  */
 export class ChatHistoryManager {
-  // Map of notebook IDs to their chat threads
+  // Map of notebook IDs to their chat threads (in-memory cache)
   private notebookChats: Map<string, IChatThread[]> = new Map();
+  // Track which notebooks have been loaded from storage
+  private loadedNotebooks: Set<string> = new Set();
   // Current active notebook ID
   private currentNotebookId: string | null = null;
   // Current active chat thread ID
   private currentThreadId: string | null = null;
-  // Storage key prefix
-  private readonly STORAGE_KEY_PREFIX = 'sage-ai-chat-history-';
+  // Storage key prefix for individual notebook chat histories
+  private readonly STORAGE_KEY_PREFIX = 'chat-history-notebook-';
   // Map of notebook IDs to their chatbox instances
   private notebookChatboxes: Map<string, NotebookChatState> = new Map();
 
   constructor() {
+    // Check for migration from old storage format
     this.loadFromStorage();
 
     // Subscribe to notebook change events from AppStateService
     AppStateService.onNotebookChanged().subscribe(
-      ({ oldNotebookId, newNotebookId }) => {
+      async ({ oldNotebookId, newNotebookId }) => {
         if (newNotebookId) {
-          this.setCurrentNotebook(newNotebookId);
+          await this.setCurrentNotebook(newNotebookId);
         }
       }
     );
@@ -64,6 +68,12 @@ export class ChatHistoryManager {
     this.notebookChats.set(newId, threads);
     this.notebookChats.delete(oldId);
 
+    // Update loaded notebooks tracking
+    if (this.loadedNotebooks.has(oldId)) {
+      this.loadedNotebooks.delete(oldId);
+      this.loadedNotebooks.add(newId);
+    }
+
     // Also update chatbox mapping
     const chatState = this.notebookChatboxes.get(oldId);
     if (chatState) {
@@ -71,7 +81,9 @@ export class ChatHistoryManager {
       this.notebookChatboxes.delete(oldId);
     }
 
-    this.saveToStorage();
+    // Save the chat history under the new ID and remove the old one
+    this.saveNotebookToStorage(newId);
+    this.removeNotebookFromStorage(oldId);
   }
 
   /**
@@ -79,7 +91,7 @@ export class ChatHistoryManager {
    * @param notebookId ID of the notebook
    * @returns The active chat thread for this notebook (creates one if none exists)
    */
-  public setCurrentNotebook(notebookId: string): IChatThread {
+  public async setCurrentNotebook(notebookId: string): Promise<IChatThread> {
     // If we're switching notebooks, hide the previous one
     if (this.currentNotebookId && this.currentNotebookId !== notebookId) {
       this.hideChatbox(this.currentNotebookId);
@@ -98,6 +110,9 @@ export class ChatHistoryManager {
     console.log(`[ChatHistoryManager] Setting current notebook: ${notebookId}`);
     this.currentNotebookId = notebookId;
 
+    // Load chat history for this notebook if not already loaded
+    await this.loadNotebookFromStorage(notebookId);
+
     // Check if we have chat history for this notebook
     if (!this.notebookChats.has(notebookId)) {
       // Create a default thread for this notebook
@@ -111,7 +126,7 @@ export class ChatHistoryManager {
       };
 
       this.notebookChats.set(notebookId, [defaultThread]);
-      this.saveToStorage();
+      await this.saveNotebookToStorage(notebookId);
     }
 
     // Get all threads for this notebook
@@ -266,8 +281,8 @@ export class ChatHistoryManager {
     threads[threadIndex].contexts = new Map(contexts);
     threads[threadIndex].lastUpdated = Date.now();
 
-    // Save to storage
-    this.saveToStorage();
+    // Save only this notebook's data to storage
+    this.saveNotebookToStorage(this.currentNotebookId);
   }
 
   /**
@@ -338,8 +353,8 @@ export class ChatHistoryManager {
       threads[threadIndex].contexts = new Map(contexts);
     }
 
-    // Save to storage
-    this.saveToStorage();
+    // Save only this notebook's data to storage
+    this.saveNotebookToStorage(this.currentNotebookId);
   }
 
   public static getCleanMessageArrayWithTimestamps(thread: IChatThread): any[] {
@@ -359,140 +374,178 @@ export class ChatHistoryManager {
   }
 
   /**
-   * Get all notebook paths with chat histories
+   * Get all notebook paths with chat histories (loaded and in storage)
    * @returns Array of notebook paths
    */
-  public getNotebookIds(): string[] {
+  public async getNotebookIds(): Promise<string[]> {
+    // Get currently loaded notebook IDs
+    const loadedIds = Array.from(this.notebookChats.keys());
+    
+    // Get all notebook IDs that have storage entries
+    try {
+      const allKeys = await StateDBCachingService.listKeys();
+      const storageIds = allKeys
+        .filter(key => key.startsWith(this.STORAGE_KEY_PREFIX))
+        .map(key => key.replace(this.STORAGE_KEY_PREFIX, ''));
+      
+      // Combine and deduplicate
+      const allIds = new Set([...loadedIds, ...storageIds]);
+      return Array.from(allIds);
+    } catch (error) {
+      console.error('[ChatHistoryManager] Error listing notebook IDs from storage:', error);
+      // Fallback to only loaded IDs
+      return loadedIds;
+    }
+  }
+
+  /**
+   * Get all notebook paths with chat histories (synchronous, only loaded notebooks)
+   * @returns Array of notebook paths currently loaded in memory
+   * @deprecated Use getNotebookIds() for complete list including storage
+   */
+  public getLoadedNotebookIds(): string[] {
     return Array.from(this.notebookChats.keys());
   }
 
   /**
-   * Save all chat histories to state database
+   * Save a specific notebook's chat history to storage
+   * @param notebookId The notebook ID to save
    */
-  private async saveToStorage(): Promise<void> {
+  private async saveNotebookToStorage(notebookId: string): Promise<void> {
+    if (!notebookId || !this.notebookChats.has(notebookId)) {
+      console.warn(`[ChatHistoryManager] Cannot save: notebook ${notebookId} not found in cache`);
+      return;
+    }
+
     try {
-      // Convert Map to a serializable object
-      const storageObj: Record<string, any[]> = {};
+      const threads = this.notebookChats.get(notebookId)!;
+      
+      // Convert each thread's contexts Map to a serializable object
+      const serializedThreads = threads.map(thread => ({
+        ...thread,
+        contexts: thread.contexts ? Object.fromEntries(thread.contexts) : {},
+        message_timestamps: thread.message_timestamps
+          ? Object.fromEntries(thread.message_timestamps)
+          : {}
+      }));
 
-      console.log('SAVING CHAT TO STORAGE');
-      console.log(this.notebookChats);
+      const storageKey = `${this.STORAGE_KEY_PREFIX}${notebookId}`;
+      await StateDBCachingService.setObjectValue(storageKey, serializedThreads);
 
-      for (const [notebookId, threads] of this.notebookChats.entries()) {
-        // Convert each thread's contexts Map to a serializable object
-        const serializedThreads = threads.map(thread => ({
-          ...thread,
-          contexts: thread.contexts ? Object.fromEntries(thread.contexts) : {},
-          message_timestamps: thread.message_timestamps
-            ? Object.fromEntries(thread.message_timestamps)
-            : {}
-        }));
-        storageObj[notebookId] = serializedThreads;
-      }
-
-      await StateDBCachingService.setObjectValue(
-        STATE_DB_KEYS.CHAT_HISTORIES,
-        storageObj
-      );
-
-      console.log(
-        '[ChatHistoryManager] Saved chat histories to StateDB storage'
-      );
+      console.log(`[ChatHistoryManager] Saved chat history for notebook: ${notebookId}`);
     } catch (error) {
-      console.error(
-        '[ChatHistoryManager] Error saving chat histories to StateDB storage:',
-        error
-      );
+      console.error(`[ChatHistoryManager] Error saving chat history for notebook ${notebookId}:`, error);
     }
   }
 
   /**
-   * Load chat histories from state database with migration from settings registry
+   * Load a specific notebook's chat history from storage
+   * @param notebookId The notebook ID to load
+   */
+  private async loadNotebookFromStorage(notebookId: string): Promise<void> {
+    if (this.loadedNotebooks.has(notebookId)) {
+      // Already loaded
+      return;
+    }
+
+    try {
+      const storageKey = `${this.STORAGE_KEY_PREFIX}${notebookId}`;
+      const storedThreads = await StateDBCachingService.getObjectValue<any[]>(storageKey, []);
+
+      if (storedThreads && storedThreads.length > 0) {
+        // Convert object back to proper IChatThread format
+        const migratedThreads: IChatThread[] = storedThreads.map(thread => ({
+          ...thread,
+          contexts: thread.contexts
+            ? new Map<string, MentionContext>(Object.entries(thread.contexts))
+            : new Map<string, MentionContext>(),
+          message_timestamps: thread.message_timestamps
+            ? new Map<string, number>(Object.entries(thread.message_timestamps))
+            : new Map<string, number>()
+        }));
+
+        this.notebookChats.set(notebookId, migratedThreads);
+        console.log(`[ChatHistoryManager] Loaded ${migratedThreads.length} threads for notebook: ${notebookId}`);
+      } else {
+        console.log(`[ChatHistoryManager] No stored chat history found for notebook: ${notebookId}`);
+      }
+
+      this.loadedNotebooks.add(notebookId);
+    } catch (error) {
+      console.error(`[ChatHistoryManager] Error loading chat history for notebook ${notebookId}:`, error);
+      this.loadedNotebooks.add(notebookId); // Mark as loaded even on error to avoid retry loops
+    }
+  }
+
+  /**
+   * Remove a notebook's chat history from storage
+   * @param notebookId The notebook ID to remove
+   */
+  private async removeNotebookFromStorage(notebookId: string): Promise<void> {
+    try {
+      const storageKey = `${this.STORAGE_KEY_PREFIX}${notebookId}`;
+      await StateDBCachingService.removeValue(storageKey);
+      console.log(`[ChatHistoryManager] Removed chat history for notebook: ${notebookId}`);
+    } catch (error) {
+      console.error(`[ChatHistoryManager] Error removing chat history for notebook ${notebookId}:`, error);
+    }
+  }
+
+  /**
+   * Save all loaded notebook histories to storage
+   * @deprecated Use saveNotebookToStorage for individual notebooks instead
+   */
+  private async saveToStorage(): Promise<void> {
+    console.warn('[ChatHistoryManager] saveToStorage is deprecated, saving all loaded notebooks individually');
+    
+    // Save each loaded notebook individually
+    for (const notebookId of this.loadedNotebooks) {
+      if (this.notebookChats.has(notebookId)) {
+        await this.saveNotebookToStorage(notebookId);
+      }
+    }
+  }
+
+  /**
+   * Load chat histories from state database (migration support)
+   * @deprecated Chat histories are now loaded per-notebook on demand
    */
   private async loadFromStorage(): Promise<void> {
+    console.log('[ChatHistoryManager] loadFromStorage is deprecated - using lazy loading per notebook');
+    
     try {
-      // First, try to migrate data from settings registry to state database
-      await this.migrateFromSettingsRegistry();
+      // Check if there's old format data that needs migration
+      const oldStoredData = await StateDBCachingService.getObjectValue<Record<string, any[]>>(
+        STATE_DB_KEYS.CHAT_HISTORIES,
+        {}
+      );
 
-      // Load data from state database
-      const storedData = await StateDBCachingService.getObjectValue<
-        Record<string, any[]>
-      >(STATE_DB_KEYS.CHAT_HISTORIES, {});
-
-      if (storedData && Object.keys(storedData).length > 0) {
-        // Convert object back to Map
-        this.notebookChats = new Map();
-        for (const [notebookId, threads] of Object.entries(storedData)) {
-          // Migrate old storage format to new format with contexts
+      if (oldStoredData && Object.keys(oldStoredData).length > 0) {
+        console.log('[ChatHistoryManager] Found old format data, migrating to new per-notebook storage...');
+        
+        // Migrate each notebook's data to individual storage
+        for (const [notebookId, threads] of Object.entries(oldStoredData)) {
           const migratedThreads: IChatThread[] = threads.map(thread => ({
             ...thread,
-            // Handle migration for threads that don't have contexts
             contexts: thread.contexts
               ? new Map<string, MentionContext>(Object.entries(thread.contexts))
               : new Map<string, MentionContext>(),
             message_timestamps: thread.message_timestamps
-              ? new Map<string, number>(
-                  Object.entries(thread.message_timestamps)
-                )
+              ? new Map<string, number>(Object.entries(thread.message_timestamps))
               : new Map<string, number>()
           }));
 
           this.notebookChats.set(notebookId, migratedThreads);
+          this.loadedNotebooks.add(notebookId);
+          await this.saveNotebookToStorage(notebookId);
         }
 
-        console.log(
-          '[ChatHistoryManager] Loaded chat histories from StateDB storage'
-        );
-        console.log(
-          `[ChatHistoryManager] Loaded ${this.notebookChats.size} notebook chat histories`
-        );
-      } else {
-        console.log(
-          '[ChatHistoryManager] No stored chat histories found in StateDB'
-        );
+        // Remove the old centralized storage
+        await StateDBCachingService.removeValue(STATE_DB_KEYS.CHAT_HISTORIES);
+        console.log('[ChatHistoryManager] Migration completed, old centralized storage removed');
       }
     } catch (error) {
-      console.error(
-        '[ChatHistoryManager] Error loading chat histories from StateDB storage:',
-        error
-      );
-      // Reset to empty state on error
-      this.notebookChats = new Map();
-    }
-  }
-
-  /**
-   * Migrate chat histories from settings registry to state database
-   */
-  private async migrateFromSettingsRegistry(): Promise<void> {
-    try {
-      // Check if data exists in settings registry
-      const settingsData = await CachingService.getObjectSetting<
-        Record<string, any[]>
-      >(SETTING_KEYS.CHAT_HISTORIES, {});
-
-      if (settingsData && Object.keys(settingsData).length > 0) {
-        console.log(
-          '[ChatHistoryManager] Migrating chat histories from SettingsRegistry to StateDB'
-        );
-
-        // Save to state database
-        await StateDBCachingService.setObjectValue(
-          STATE_DB_KEYS.CHAT_HISTORIES,
-          settingsData
-        );
-
-        // Clear from settings registry
-        await CachingService.setObjectSetting(SETTING_KEYS.CHAT_HISTORIES, {});
-
-        console.log(
-          '[ChatHistoryManager] Successfully migrated chat histories to StateDB'
-        );
-      }
-    } catch (error) {
-      console.error(
-        '[ChatHistoryManager] Error during migration from SettingsRegistry to StateDB:',
-        error
-      );
+      console.error('[ChatHistoryManager] Error during migration from old storage format:', error);
     }
   }
 
@@ -500,9 +553,7 @@ export class ChatHistoryManager {
    * Generate a unique ID for a new chat thread
    */
   private generateThreadId(): string {
-    return (
-      'thread-' + Date.now() + '-' + Math.random().toString(36).substring(2, 9)
-    );
+    return 'thread_' + uuidv4();
   }
 
   /**
@@ -538,8 +589,10 @@ export class ChatHistoryManager {
     // Set the new thread as the current thread
     this.currentThreadId = newThread.id;
 
-    // Save to storage
-    this.saveToStorage();
+    // Save only this notebook's data to storage
+    if (this.currentNotebookId) {
+      this.saveNotebookToStorage(this.currentNotebookId);
+    }
 
     return newThread;
   }
@@ -602,8 +655,10 @@ export class ChatHistoryManager {
     // Update the thread name
     threads[threadIndex].name = newName;
 
-    // Save to storage
-    this.saveToStorage();
+    // Save only this notebook's data to storage
+    if (this.currentNotebookId) {
+      this.saveNotebookToStorage(this.currentNotebookId);
+    }
 
     console.log(
       `[ChatHistoryManager] Renamed thread ${this.currentThreadId} to "${newName}"`
@@ -658,10 +713,82 @@ export class ChatHistoryManager {
       }
     }
 
-    // Save to storage
-    this.saveToStorage();
+    // Save only this notebook's data to storage
+    if (this.currentNotebookId) {
+      this.saveNotebookToStorage(this.currentNotebookId);
+    }
 
     console.log(`[ChatHistoryManager] Deleted thread ${threadId}`);
     return true;
   }
+
+  /**
+   * Clear all chat history for a specific notebook
+   * @param notebookId The notebook ID to clear
+   * @returns True if successful, false otherwise
+   */
+  public async clearNotebookHistory(notebookId: string): Promise<boolean> {
+    try {
+      // Remove from memory cache
+      this.notebookChats.delete(notebookId);
+      this.loadedNotebooks.delete(notebookId);
+      
+      // Remove from storage
+      await this.removeNotebookFromStorage(notebookId);
+      
+      // If this was the current notebook, reset current state
+      if (this.currentNotebookId === notebookId) {
+        this.currentNotebookId = null;
+        this.currentThreadId = null;
+      }
+      
+      console.log(`[ChatHistoryManager] Cleared all chat history for notebook: ${notebookId}`);
+      return true;
+    } catch (error) {
+      console.error(`[ChatHistoryManager] Error clearing chat history for notebook ${notebookId}:`, error);
+      return false;
+    }
+  }
+
+  /**
+   * Unload a notebook's chat history from memory (but keep in storage)
+   * @param notebookId The notebook ID to unload
+   */
+  public unloadNotebookFromMemory(notebookId: string): void {
+    if (notebookId === this.currentNotebookId) {
+      console.warn(`[ChatHistoryManager] Cannot unload current notebook ${notebookId} from memory`);
+      return;
+    }
+
+    this.notebookChats.delete(notebookId);
+    this.loadedNotebooks.delete(notebookId);
+    console.log(`[ChatHistoryManager] Unloaded notebook ${notebookId} from memory`);
+  }
+
+  /**
+   * Get cache statistics
+   * @returns Object with cache information
+   */
+  public getCacheStats(): {
+    loadedNotebooks: number;
+    totalThreadsInMemory: number;
+    currentNotebook: string | null;
+    memoryUsageEstimate: string;
+  } {
+    let totalThreads = 0;
+    let memorySize = 0;
+
+    for (const [notebookId, threads] of this.notebookChats.entries()) {
+      totalThreads += threads.length;
+      // Rough estimate of memory usage
+      memorySize += JSON.stringify(threads).length;
+    }
+
+    return {
+      loadedNotebooks: this.notebookChats.size,
+      totalThreadsInMemory: totalThreads,
+      currentNotebook: this.currentNotebookId,
+      memoryUsageEstimate: `${(memorySize / 1024).toFixed(2)} KB`
+    };
+  }
 }
