@@ -607,7 +607,7 @@ export class ConversationService {
     const allActions: IActionHistoryEntry[] = [];
     for (const cp of checkpointsToRedo) {
       if (cp.actionHistory && cp.actionHistory.length > 0) {
-        allActions.push(...cp.actionHistory);
+        allActions.push(...cp.actionHistory.slice().reverse());
       }
     }
 
@@ -660,10 +660,11 @@ export class ConversationService {
       id: 'redo_add_cell',
       name: 'notebook-add_cell',
       input: {
-        cell_type: 'code', // Default to code cell type
+        cell_type: action.data.originalCellType, // Default to code cell type
         source: action.data.newContent || action.data.source || '',
         summary: action.data.summary || 'Redone by checkpoint restoration',
-        tracking_id: trackingId // Reuse the same tracking ID
+        tracking_id: trackingId, // Reuse the same tracking ID
+        run_cell: action.data.originalCellType === 'markdown'
       }
     });
   }
@@ -681,7 +682,7 @@ export class ConversationService {
       name: 'notebook-edit_cell',
       input: {
         cell_id: trackingId,
-        new_source: action.data.newSource,
+        new_source: action.data.newContent,
         summary: action.data.summary || 'Redone by checkpoint restoration',
         is_tracking_id: true
       }
@@ -692,14 +693,13 @@ export class ConversationService {
    * Redo removing cells
    */
   private async redoRemoveCells(action: IActionHistoryEntry): Promise<void> {
-    // Remove the cells again using the same cell IDs
-    const cellIds = action.data.removedCells?.map(cell => cell.trackingId);
-    if (cellIds && cellIds.length > 0) {
+    const cellId = action.data.cellId;
+    if (cellId) {
       await this.toolService.executeTool({
         id: 'redo_remove_cells',
         name: 'notebook-remove_cells',
         input: {
-          cell_ids: cellIds,
+          cell_ids: [cellId],
           remove_from_notebook: true
         }
       });
@@ -949,25 +949,20 @@ export class ConversationService {
    * Undo removing cells
    */
   private async undoRemoveCells(action: IActionHistoryEntry): Promise<void> {
-    // Re-add each removed cell
-    if (action.data.removedCells) {
-      for (let i = 0; i < action.data.removedCells.length; i++) {
-        const cell = action.data.removedCells[i];
-        // Generate a tracking ID if none was saved
-        const trackingId = cell.trackingId || `restored-${Date.now()}-${i}`;
-
-        await this.toolService.executeTool({
-          id: 'undo_remove_cell',
-          name: 'notebook-add_cell',
-          input: {
-            cell_type: cell.type,
-            source: cell.content,
-            summary: cell.custom?.summary || 'Restored by undo',
-            position: cell.custom?.index, // Use index from custom metadata if available
-            tracking_id: trackingId // Provide tracking ID to reuse
-          }
-        });
-      }
+    const cellId = action.data.cellId;
+    if (cellId) {
+      const metadata = action.data.metadata;
+      await this.toolService.executeTool({
+        id: 'undo_remove_cell',
+        name: 'notebook-add_cell',
+        input: {
+          cell_type: 'code',
+          source: action.data.oldContent,
+          summary: metadata.custom?.summary || 'Restored by undo',
+          position: metadata.custom?.index, // Use index from custom metadata if available
+          tracking_id: cellId // Provide tracking ID to reuse
+        }
+      });
     }
   }
 
