@@ -1188,30 +1188,7 @@ export class ConversationServiceUtils {
     toolResult: any,
     actionBeforeExecution: any
   ): void {
-    if (toolName === 'notebook-add_cell' && toolResult && toolResult.content) {
-      try {
-        const cellId = toolResult.content.replace(/"/g, '').trim();
-        context.actionHistory.addActionWithCheckpoint(
-          ActionType.ADD_CELL,
-          { cellId, ...toolArgs },
-          `Added ${toolArgs.cell_type} cell`
-        );
-      } catch (err) {
-        console.error('Failed to track add_cell action:', err);
-      }
-    } else if (toolName === 'notebook-edit_cell' && actionBeforeExecution) {
-      context.actionHistory.addActionWithCheckpoint(
-        ActionType.EDIT_CELL,
-        actionBeforeExecution,
-        `Edited cell ${toolArgs.cell_id.substring(0, 8)}...`
-      );
-    } else if (toolName === 'notebook-remove_cells' && actionBeforeExecution) {
-      context.actionHistory.addActionWithCheckpoint(
-        ActionType.REMOVE_CELLS,
-        actionBeforeExecution,
-        `Removed ${toolArgs.cell_ids.length} cell(s)`
-      );
-
+    if (toolName === 'notebook-remove_cells' && actionBeforeExecution) {
       // Track remove diffs if we have a diff manager
       if (context.diffManager && actionBeforeExecution.removedCells) {
         for (const cell of actionBeforeExecution.removedCells) {
