@@ -2,6 +2,7 @@ import { LabIcon } from '@jupyterlab/ui-components';
 import menuIcon from '../../../style/icons/state_display/menu-icon.svg';
 import menuCloseIcon from '../../../style/icons/state_display/menu-close.svg';
 import runCellIcon from '../../../style/icons/state_display/run_cell.svg';
+import warningIcon from '../../../style/icons/state_display/warning.svg';
 
 export const MENU_ICON = new LabIcon({
   name: 'sage-agent-internal:state-menu-icon', // unique name for your icon
@@ -17,3 +18,8 @@ export const RUN_CELL_ICON = new LabIcon({
   name: 'sage-agent-internal:run-cell-icon',
   svgstr: runCellIcon
 });
+
+export const WARNING_ICON = new LabIcon({
+  name: 'sage-agent-internal:warning-icon',
+  svgstr: warningIcon
+});
